package mon.stat

import scala.io.Source

import Statistic._

import mon.util.YamlHelper

abstract class Form {
  val name : String
  //val height
  //val weight
  val desc : String
  val elements : IndexedSeq[Element]
  val baseStats : Map[Stat, Int]
  // val appearance // animation
  // moves
  // abilities
  val catchRate : Int
  // val color
  
  override def toString = {
    name
  }
}

case class FormToken(
    val name : String,
    val desc : String,
    val elements : IndexedSeq[String],
    val baseStats : Map[String, Int],
    val catchRate : Int = 255
    ) {
  
  def instantiate() = {
    val self = this
    new Form {
      val name = self.name
      val desc = self.desc
      val elements = self.elements.map(Element(_))
      val baseStats = self.baseStats.map{ case (s, i) => (Statistic(s), i)}
      val catchRate = self.catchRate
    }
  }
}

object Form {
  def apply(name : String) = byName(name)
  
  def fromMap(dict : Map[String, Any]) = {
    new Form {
      val name = dict("name").toString
      val desc = dict("desc").toString()
      val elements = IndexedSeq()
      val baseStats = Statistic.buildMap(_ => 10)
      val catchRate = 255
    }
  }
  
  val forms = YamlHelper.extractSeq[FormToken](Form.getClass.getResourceAsStream("data/forms.yaml")).map(_.instantiate())
  val byName = forms.map(f => (f.name, f)).toMap
}